@@ -3,8 +3,6 @@
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from __future__ import unicode_literals
-
 """Tools for diffing notebooks.
 
 All diff tools here currently assumes the notebooks have already been
@@ -12,14 +10,20 @@ converted to the same format version, currently v4 at time of writing.
 Up- and down-conversion is handled by nbformat.
 """
 
+from __future__ import unicode_literals
+
 import operator
 import re
 import copy
 from collections import defaultdict
 from six import string_types
 from six.moves import zip
+try:
+    from functools import lru_cache
+except ImportError:
+    from backports.functools_lru_cache import lru_cache
 
-from ..diff_format import source_as_string, MappingDiffBuilder, DiffOp
+from ..diff_format import MappingDiffBuilder, DiffOp
 
 from .generic import (diff, diff_sequence_multilevel,
                       compare_strings_approximate)
@@ -27,7 +31,7 @@ from .generic import (diff, diff_sequence_multilevel,
 __all__ = ["diff_notebooks"]
 
 # A regexp matching base64 encoded data
-_base64 = re.compile(r'^(?:[a-z0-9+/]{4})*(?:[a-z0-9+/]{2}==|[a-z0-9+/]{3}=)?$', re.MULTILINE | re.UNICODE | re.IGNORECASE)
+_base64 = re.compile(r'^(?:[a-z0-9+/]{4})*(?:[a-z0-9+/]{2}==|[a-z0-9+/]{3}=)?$', re.UNICODE | re.IGNORECASE)
 
 # A regexp matching common python repr-style output like
 # <module.type at 0xmemoryaddress>
@@ -45,10 +49,18 @@ _split_mimes = (
     )
 
 
+def _is_base64(test_string, min_len=64):
+    """Whether string is base64 data, possibly with newlines in it"""
+    if len(test_string) < min_len:
+        return False
+    return _base64.match(''.join(test_string.splitlines()))
+
+
 # TODO: Maybe cleaner to make the split between strict/approximate
 #       an argument instead of separate functions.
 
 
+@lru_cache(maxsize=1024, typed=False)
 def compare_text_approximate(x, y):
     # Fast cutoff when one is empty
     if bool(x) != bool(y):
@@ -90,6 +102,15 @@ def compare_base64_strict(x, y):
     return x == y
 
 
+@lru_cache(maxsize=128, typed=False)
+def _compare_mimedata_strings(x, y, comp_text, comp_base64):
+    # Most likely base64 encoded data
+    if _is_base64(x):
+        return comp_base64(x, y)
+    else:
+        return comp_text(x, y)
+
+
 def _compare_mimedata(mimetype, x, y, comp_text, comp_base64):
     mimetype = mimetype.lower()
 
@@ -108,12 +129,7 @@ def _compare_mimedata(mimetype, x, y, comp_text, comp_base64):
     # TODO: Compare binary images?
     #if mimetype.startswith("image/"):
     if isinstance(x, string_types) and isinstance(y, string_types):
-        # Most likely base64 encoded data
-        if _base64.match(x):
-            return comp_base64(x, y)
-        else:
-            return comp_text(x, y)
-
+        _compare_mimedata_strings(x, y, comp_text, comp_base64)
     # Fallback to exactly equal
     return x == y
 
@@ -283,6 +299,7 @@ def compare_cell_approximate(x, y):
 
 
 def compare_outputs_approximate(xoutputs, youtputs):
+    """Compare sequences of outputs with approximate heuristics."""
     dd = diff_item_at_path(xoutputs, youtputs, "/cells/*/outputs")
     if any(e.op != DiffOp.PATCH for e in dd):
         # Something added or removed
@@ -350,24 +367,27 @@ def compare_cell_strict(x, y):
 
 def diff_single_outputs(a, b, path="/cells/*/outputs/*",
                         predicates=None, differs=None):
-    "DiffOp a pair of output cells."
-    assert path == "/cells/*/outputs/*"
-    assert a.output_type == b.output_type
+    """DiffOp a pair of output cells."""
+    assert path == "/cells/*/outputs/*", 'Invalid path for ouput: %r' % path
+    assert a.output_type == b.output_type, 'cannot diff outputs of different types'
 
     if a.output_type in ("execute_result", "display_data"):
         di = MappingDiffBuilder()
 
+        # Separate data from output during diffing:
         tmp_data = a.pop('data')
-        a_conj = copy.deepcopy(a)
-        a.data = tmp_data
+        a_conj = copy.deepcopy(a)  # Output without data
+        a.data = tmp_data          # Restore output
         tmp_data = b.pop('data')
         b_conj = copy.deepcopy(b)
         b.data = tmp_data
+        # Only diff outputs without data:
         dd_conj = diff(a_conj, b_conj)
         if dd_conj:
             for e in dd_conj:
                 di.append(e)
 
+        # Only diff data:
         dd = diff_mime_bundle(a.data, b.data, path=path+"/data")
         if dd:
             di.patch("data", dd)
@@ -393,7 +413,7 @@ def add_mime_diff(key, avalue, bvalue, diffbuilder):
 def diff_attachments(a, b, path="/cells/*/attachments",
                      predicates=None, differs=None):
     """Diff a pair of attachment collections"""
-    assert path == "/cells/*/attachments"
+    assert path == "/cells/*/attachments", 'Invalid path for attachment: %r' % path
 
     # Two events can happen:
     #  1: An attachment is added/removed/patched
@@ -401,7 +421,8 @@ def diff_attachments(a, b, path="/cells/*/attachments",
     # Currently, #2 is handled as two ops (an add and a remove)
 
     # keys here are 'filenames' of the attachments
-    assert isinstance(a, dict) and isinstance(b, dict)
+    if not isinstance(a, dict) or not isinstance(b, dict):
+        raise TypeError('Attachments stores should be dictionaries. Got %r and %r' % (a, b))
     akeys = set(a.keys())
     bkeys = set(b.keys())
 
@@ -427,8 +448,17 @@ def diff_attachments(a, b, path="/cells/*/attachments",
 
 def diff_mime_bundle(a, b, path=None,
                      predicates=None, differs=None):
+    """Diff a MIME bundle.
+
+    A MIME bundle has MIME types as keys, with values that are
+    of the corresponding MIME type.
+
+    Very similar to the generic dict differ, except it passes
+    the diff builder to the sub-differ.
+    """
     # keys here are mime/types
-    assert isinstance(a, dict) and isinstance(b, dict)
+    if not isinstance(a, dict) or not isinstance(b, dict):
+        raise TypeError('MIME bundles should be dictionaries. Got %r and %r' % (a, b))
     akeys = set(a.keys())
     bkeys = set(b.keys())
 
@@ -449,6 +479,27 @@ def diff_mime_bundle(a, b, path=None,
     return di.validated()
 
 
+def diff_ignore(*args, **kwargs):
+    """Always returns an empty diff"""
+    return []
+
+
+def diff_ignore_keys(inner_differ, ignore_keys):
+    """Call inner_differ, but filter the resulting diff.
+
+    Will ignore all direct diff values that has a key in
+    ignore_keys. I.e. this will not recurse into patch ops.abs
+    """
+    def ignored_diff(*args, **kwargs):
+        d = inner_differ(*args, **kwargs)
+        ret = []
+        for e in d:
+            if e.key not in ignore_keys:
+                ret.append(e)
+        return ret
+    return ignored_diff
+
+
 # Sequence diffs should be applied with multilevel
 # algorithm for paths with more than one predicate,
 # and using operator.__eq__ if no match in there.
@@ -476,6 +527,43 @@ notebook_differs = defaultdict(lambda: diff, {
     })
 
 
+def set_notebook_diff_targets(sources=True, outputs=True, attachments=True, metadata=True, details=True):
+    """Configure the notebook differs to include/ignore various changes."""
+    if sources:
+        if "/cells/*/source" in notebook_differs:
+            del notebook_differs["/cells/*/source"]
+    else:
+        notebook_differs["/cells/*/source"] = diff_ignore
+
+    if outputs:
+        notebook_differs["/cells/*/outputs"] = diff_sequence_multilevel
+    else:
+        notebook_differs["/cells/*/outputs"] = diff_ignore
+
+    if attachments:
+        notebook_differs["/cells/*/attachments"] = diff_attachments
+    else:
+        notebook_differs["/cells/*/attachments"] = diff_ignore
+
+    metadata_keys = ("/cells/*/metadata", "/metadata", "/cells/*/outputs/*/metadata")
+    if metadata:
+        for key in metadata_keys:
+            if key in notebook_differs:
+                del notebook_differs[key]
+    else:
+        for key in metadata_keys:
+            notebook_differs[key] = diff_ignore
+
+    if details:
+        notebook_differs['/cells/*'] = diff
+        notebook_differs["/cells/*/outputs/*"] = diff_single_outputs
+    else:
+        notebook_differs['/cells/*'] = diff_ignore_keys(
+            inner_differ=diff, ignore_keys=['execution_count'])
+        notebook_differs['/cells/*/outputs/*'] = diff_ignore_keys(
+            inner_differ=diff_single_outputs, ignore_keys=['execution_count'])
+
+
 def diff_cells(a, b):
     "This is currently just used by some tests."
     path = "/cells"
@@ -483,6 +571,7 @@ def diff_cells(a, b):
 
 
 def diff_item_at_path(a, b, path):
+    """Calculate the diff using the configured notebook differ for path."""
     return notebook_differs[path](a, b, path=path, predicates=notebook_predicates, differs=notebook_differs)
 
 
