@@ -5,24 +5,27 @@
 
 from __future__ import unicode_literals
 
-import nbformat
 from six import string_types
-import copy
 from collections import defaultdict
-from itertools import chain
 
+import nbdime.log
 from .decisions import MergeDecisionBuilder
 from .chunks import make_merge_chunks
-from ..diffing import diff
+from .strategies import (
+    resolve_strategy_generic,
+     resolve_conflicted_decisions_dict,
+    resolve_conflicted_decisions_list,
+    resolve_conflicted_decisions_strings,
+    resolve_strategy_inline_source,
+)
+from ..diffing import diff as perform_diff
 from ..diff_format import (
-    DiffEntry, DiffOp, ParentDeleted, Missing, as_dict_based_diff,
-    op_patch, op_addrange, op_removerange, op_add, op_remove, op_replace)
+    DiffEntry, DiffOp, ParentDeleted, Missing,
+    op_patch, op_addrange, op_removerange)
+from ..diff_utils import as_dict_based_diff
 from ..diffing.notebooks import notebook_predicates, notebook_differs
 from ..patching import patch
-from ..utils import star_path, join_path, Strategies
-import nbdime.log
-from nbdime.prettyprint import merge_render
-from nbdime.merging.decisions import pop_all_patch_decisions, push_patch_decision
+from ..utils import star_path, Strategies
 
 
 # =============================================================================
@@ -78,7 +81,7 @@ def will_diff_counter_parent_deletion(diff, path, strategies):
 
 
 def create_parent_deletion_counter_diff(diff, path, strategies):
-    ""
+    """Create diff for when a parent deletion is countered by strategy"""
     newdiff = []
     # Resolve diff paths and check them vs transients list
     for d in diff:
@@ -111,46 +114,6 @@ def split_diffs_by_index(diffs):
     return diffs_by_index
 
 
-def bundle_decisions_by_index(base_path, decisions):
-    decisions_by_index = defaultdict(list)
-    level = len(base_path)
-    for d in decisions:
-        assert base_path == d.common_path[:level]
-        if len(d.common_path) > level:
-            # At least patch/patch will have common_path on a particular item
-            key = d.common_path[level]
-            # Wrap decision diffs in patches so common_path points to list
-            prefix = d.common_path[level:]
-            d = push_patch_decision(d, prefix)
-        else:
-            # Removerange or addrange will have common_path
-            # on list and key only in the diff entries
-            keys = set(e.key for e in chain(d.local_diff, d.remote_diff, d.get("custom_diff", ())))
-            assert len(keys) == 1
-            key, = keys
-        decisions_by_index[key].append(d)
-    return decisions_by_index
-
-
-def combine_patches(diffs):
-    """Rewrite diffs in canonical form where only one patch
-    applies to one key and diff entries are sorted by key."""
-    patches = {}
-    newdiffs = []
-    for d in diffs:
-        if d.op == DiffOp.PATCH:
-            p = patches.get(d.key)
-            if p is None:
-                p = op_patch(d.key, combine_patches(d.diff))
-                newdiffs.append(p)
-                patches[d.key] = p
-            else:
-                p.diff = combine_patches(p.diff + d.diff)
-        else:
-            newdiffs.append(d)
-    return sorted(newdiffs, key=lambda x: x.key)
-
-
 def __unused__wrap_subconflicts(key, subconflicts):
     "Wrap diffs in conflict tuples in patches with given key."
     unresolved_conflicts = []
@@ -185,78 +148,19 @@ def __unused__tryresolve_conflicts(conflicts, decisions):
     return unresolved_conflicts
 
 
-def adjust_patch_level(target_path, common_path, diff):
-    n = len(target_path)
-    assert common_path[:n] == target_path
-    if n == len(target_path):
-        return diff
-    remainder_path = tuple(reversed(common_path[n:]))
-    newdiff = []
-    for d in diff:
-        nd = d
-        assert nd is not None
-        for key in remainder_path:
-            nd = op_patch(key, nd)
-        newdiff.append(nd)
-    return newdiff
+def chunk_typename(diffs):
+    """For the diffs of a chunk, return string representations of its type.
 
+    Returns a two-tuple of strings.
+     - The first string contains zero or more characters 'a' and/or 'A', where
+       'a' signifies a dict 'add' operation, and 'A' signifies a sequence
+       'addrange' operation.
+    - The second string contains zero or more characters 'P', 'R', 'r',
+      and/or 'c': 'P': patch, 'R': removerange, 'r': remove, 'c': replace
 
-def collect_diffs(path, decisions):
-    local_diff = []
-    remote_diff = []
-    for d in decisions:
-        ld = adjust_patch_level(path, d.common_path, d.local_diff)
-        rd = adjust_patch_level(path, d.common_path, d.remote_diff)
-        local_diff.extend(ld)
-        remote_diff.extend(rd)
-    local_diff = combine_patches(local_diff)
-    remote_diff = combine_patches(remote_diff)
-    return local_diff, remote_diff
-
-
-def collect_conflicting_diffs(path, decisions):
-    local_conflict_diffs = []
-    remote_conflict_diffs = []
-    for d in decisions:
-        if d.conflict:
-            ld = adjust_patch_level(path, d.common_path, d.local_diff)
-            rd = adjust_patch_level(path, d.common_path, d.remote_diff)
-            local_conflict_diffs.extend(ld)
-            remote_conflict_diffs.extend(rd)
-    return local_conflict_diffs, remote_conflict_diffs
-
-
-def collect_unresolved_diffs(base_path, unresolved_conflicts):
-    """Collect local and remote diffs from conflict tuples.
-
-    Assumed to be on the same path."""
-
-    # Collect diffs
-    local_conflict_diffs = []
-    remote_conflict_diffs = []
-    for conf in unresolved_conflicts:
-        (path, ld, rd, strategy) = conf
-
-        assert base_path == path
-        #assert base_path == path[:len(base_path)]
-        #relative_path = path[len(base_path):]
-        #assert not relative_path
-
-        assert isinstance(ld, list)
-        assert isinstance(rd, list)
-
-        for d in ld:
-            local_conflict_diffs.append(d)
-        for d in rd:
-            remote_conflict_diffs.append(d)
-
-    # Combine patches into canonical tree again
-    local_conflict_diffs = combine_patches(local_conflict_diffs)
-    remote_conflict_diffs = combine_patches(remote_conflict_diffs)
-    return local_conflict_diffs, remote_conflict_diffs
-
-
-def chunkname(diffs):
+    For a proper chunk, each string should have either 0 or 1 character each,
+    but this function will not perform any checks.
+    """
     aname = ""
     pname = ""
     for e in diffs:
@@ -275,412 +179,6 @@ def chunkname(diffs):
     return aname, pname
 
 
-# =============================================================================
-#
-# Autoresolve code follows
-#
-# =============================================================================
-
-def resolve_strategy_inline_attachments(base_path, attachments, decisions):
-    strategy = "inline-attachments"
-
-    local_conflict_diffs, remote_conflict_diffs = collect_conflicting_diffs(base_path, decisions)
-
-    # Drop conflict decisions
-    decisions.decisions = [d for d in decisions if not d.conflict]
-
-    # FIXME: Review this code.
-
-    ldiffs_by_key = {d.key: d for d in local_conflict_diffs}
-    rdiffs_by_key = {d.key: d for d in remote_conflict_diffs}
-    conflict_keys = sorted(set(ldiffs_by_key) | set(rdiffs_by_key))
-
-    for key in conflict_keys:
-        # key is the attachment filename
-        ld = ldiffs_by_key[key]
-        rd = rdiffs_by_key[key]
-
-        if ld.op == DiffOp.REMOVE:
-            # If one side is removing and we have a conflict,
-            # the other side did an edit and we keep that
-            # but flag a conflict (TODO: Or don't flag conflict?)
-            assert rd.op != DiffOp.REMOVE
-            decisions.remote(base_path, ld, rd, conflict=True, strategy=strategy)
-        elif rd.op == DiffOp.REMOVE:
-            decisions.local(base_path, ld, rd, conflict=True, strategy=strategy)
-        else:
-            # Not merging attachment contents, but adding attachments
-            # with new names LOCAL_oldname and REMOTE_oldname instead.
-
-            base = attachments[key]
-
-            if ld.op == DiffOp.ADD:
-                assert rd.op == DiffOp.ADD
-                local = ld.value
-            elif ld.op == DiffOp.REPLACE:
-                local = ld.value
-            else:
-                assert ld.op == DiffOp.PATCH
-                local = patch(base, ld.diff)
-
-            if rd.op == DiffOp.ADD:
-                remote = rd.value
-            elif rd.op == DiffOp.REPLACE:
-                remote = rd.value
-            else:
-                assert rd.op == DiffOp.PATCH
-                remote = patch(base, rd.diff)
-
-            local_name = "LOCAL_" + key
-            remote_name = "REMOTE_" + key
-
-            custom_diff = []
-
-            if local_name in attachments:
-                nbdime.log.warning("Replacing previous conflicted attachment with filename '{}'".format(local_name))
-                custom_diff += [op_replace(local_name, local)]
-            else:
-                custom_diff += [op_add(local_name, local)]
-
-            if remote_name in attachments:
-                nbdime.log.warning("Replacing previous conflicted attachment with filename '{}'".format(remote_name))
-                custom_diff += [op_replace(remote_name, remote)]
-            else:
-                custom_diff += [op_add(remote_name, remote)]
-
-            decisions.custom(base_path, ldiffs, rdiffs, custom_diff, conflict=True, strategy=strategy)
-
-
-def output_marker(text):
-    return nbformat.v4.new_output("stream", name="stderr", text=text)
-
-
-def get_outputs_and_note(base, removes, patches):
-    if removes:
-        note = " <removed>"
-        suboutputs = []
-    elif patches:
-        e, = patches  # 0 or 1 item
-
-        # Collect which mime types are modified
-        mkeys = set()
-        keys = set()
-        for d in e.diff:
-            if d.key == "data":
-                assert d.op == DiffOp.PATCH
-                for f in d.diff:
-                    mkeys.add(f.key)
-            else:
-                keys.add(d.key)
-        data = base.get("data")
-        if data:
-            ukeys = set(data.keys()) - mkeys
-        else:
-            ukeys = ()
-
-        notes = []
-        if mkeys or keys:
-            notes.append("modified: {}".format(", ".join(sorted(mkeys))))
-        if ukeys:
-            notes.append("unchanged: {}".format(", ".join(sorted(ukeys))))
-        if notes:
-            note = " <" + "; ".join(notes) + ">"
-        else:
-            note = ""
-
-        suboutputs = [patch(base, e.diff)]
-    else:
-        note = " <unchanged>"
-        suboutputs = [base]
-    return suboutputs, note
-
-
-def make_inline_output_conflict(base_output, local_diff, remote_diff):
-    """Make a list of outputs with conflict markers from conflicting
-    local and remote diffs applying to a single base_output"""
-    # Styling details
-    base_title = "base"
-    local_title = "local"
-    remote_title = "remote"
-    marker_size = 7  # default in git
-    m0 = "<"*marker_size
-    m1 = "|"*marker_size
-    m2 = "="*marker_size
-    m3 = ">"*marker_size
-
-    # Split diffs by type
-    d0 = local_diff
-    d1 = remote_diff
-    lpatches = [e for e in d0 if e.op == DiffOp.PATCH]
-    rpatches = [e for e in d1 if e.op == DiffOp.PATCH]
-    linserts = [e for e in d0 if e.op == DiffOp.ADDRANGE]
-    rinserts = [e for e in d1 if e.op == DiffOp.ADDRANGE]
-    lremoves = [e for e in d0 if e.op == DiffOp.REMOVERANGE]
-    rremoves = [e for e in d1 if e.op == DiffOp.REMOVERANGE]
-    assert len(lpatches) + len(linserts) + len(lremoves) == len(d0)
-    assert len(rpatches) + len(rinserts) + len(rremoves) == len(d1)
-
-    # Collect new outputs with surrounding markers
-    outputs = []
-
-    # Collect inserts from both sides separately
-    if linserts or rinserts:
-        lnote = ""
-        loutputs = []
-        for e in linserts:  # 0 or 1 item
-            loutputs.extend(e.valuelist)
-        rnote = ""
-        routputs = []
-        for e in rinserts:  # 0 or 1 item
-            routputs.extend(e.valuelist)
-
-        outputs.append(output_marker("%s %s%s\n" % (m0, local_title, lnote)))
-        outputs.extend(loutputs)
-        outputs.append(output_marker("%s\n" % (m2,)))
-        outputs.extend(routputs)
-        outputs.append(output_marker("%s %s%s\n" % (m3, remote_title, rnote)))
-
-    # Keep base output if untouched (only inserts)
-    keep_base = not (lremoves or rremoves or lpatches or rpatches)
-    if lremoves and rremoves:
-        # Don't add anything
-        pass
-    elif not keep_base:
-        assert not (lremoves and lpatches)
-        assert not (rremoves and rpatches)
-        lnote = ""
-        rnote = ""
-
-        # Insert changed output with surrounding markers
-        loutputs, lnote = get_outputs_and_note(base_output, lremoves, lpatches)
-        routputs, rnote = get_outputs_and_note(base_output, rremoves, rpatches)
-
-        outputs.append(output_marker("%s %s%s\n" % (m0, local_title, lnote)))
-        outputs.extend(loutputs)
-        outputs.append(output_marker("%s\n" % (m2,)))
-        outputs.extend(routputs)
-        outputs.append(output_marker("%s %s%s\n" % (m3, remote_title, rnote)))
-
-    # Return marked up output
-    return outputs, keep_base
-
-
-def resolve_strategy_remove_outputs(base_path, outputs, decisions):
-    strategy = "remove"
-
-    decisions_by_index = bundle_decisions_by_index(base_path, decisions)
-    decisions.decisions = []
-    for key, decs in sorted(decisions_by_index.items()):
-        if not any(d.conflict for d in decs):
-            decisions.decisions.extend(decs)
-        else:
-            # Replace all decisions affecting key with resolution
-            local_diff, remote_diff = collect_diffs(base_path, decs)
-            if (
-                len(local_diff) == len(remote_diff) == 1 and
-                local_diff[0].op == remote_diff[0].op == DiffOp.ADDRANGE
-            ):
-                # remove in add vs add is a no-op
-                custom_diff = []
-            else:
-                custom_diff = [op_removerange(key, 1)]
-            decisions.custom(base_path, local_diff, remote_diff,
-                custom_diff, conflict=False, strategy=strategy)
-
-
-def resolve_strategy_inline_outputs(base_path, outputs, decisions):
-    strategy = "inline-outputs"
-
-    decisions_by_index = bundle_decisions_by_index(base_path, decisions)
-    decisions.decisions = []
-    for key, decs in sorted(decisions_by_index.items()):
-        if not any(d.conflict for d in decs):
-            decisions.decisions.extend(decs)
-        else:
-            # Replace all decisions affecting key with resolution
-            local_diff, remote_diff = collect_diffs(base_path, decs)
-            inlined_conflict, keep_base = make_inline_output_conflict(
-                outputs[key], local_diff, remote_diff)
-            custom_diff = []
-            custom_diff += [op_addrange(key, inlined_conflict)]
-            if not keep_base:
-                custom_diff += [op_removerange(key, 1)]
-            decisions.custom(base_path, local_diff, remote_diff, custom_diff, conflict=True, strategy=strategy)
-
-
-def resolve_strategy_record_conflicts(base_path, base, decisions):
-    strategy = "record-conflict"
-
-    decisions.decisions = [push_patch_decision(d, d.common_path[len(base_path):]) for d in decisions]
-
-    local_conflict_diffs, remote_conflict_diffs = collect_conflicting_diffs(base_path, decisions)
-    #local_diff, remote_diff = collect_diffs(base_path, decisions)
-
-    local_conflict_diffs = combine_patches(local_conflict_diffs)
-    remote_conflict_diffs = combine_patches(remote_conflict_diffs)
-
-    # Drop conflict decisions
-    conflict_decisions = [d for d in decisions if d.conflict]
-    decisions.decisions = [d for d in decisions if not d.conflict]
-
-    # Record remaining conflicts in field nbdime-conflicts
-    conflicts_dict = {
-        "local_diff": local_conflict_diffs,
-        "remote_diff": remote_conflict_diffs,
-        # TODO: Record local and remote versions of full metadata, easier to manually select one?
-        #"base_metadata": base,
-        #"local_metadata": patch(base, local_diff),
-        #"remote_metadata": patch(base, remote_diff),
-    }
-    if "nbdime-conflicts" in base:
-        nbdime.log.warning("Replacing previous nbdime-conflicts field from base notebook.")
-        op = op_replace("nbdime-conflicts", conflicts_dict)
-    else:
-        nbdime.log.warning("Recording unresolved conflicts in {}/nbdime-conflicts.".format(join_path(base_path)))
-        op = op_add("nbdime-conflicts", conflicts_dict)
-    custom_diff = [op]
-
-    decisions.custom(base_path, local_conflict_diffs, remote_conflict_diffs, custom_diff, conflict=True, strategy=strategy)
-
-
-def resolve_strategy_inline_source(path, base, local_diff, remote_diff):
-    strategy = "inline-source"
-
-    decisions = MergeDecisionBuilder()
-
-    if local_diff is ParentDeleted:
-        # Add marker at top of cell, easier to clean up manually
-        local_diff = [op_addrange(0, ["<<<<<<< LOCAL CELL DELETED >>>>>>>\n"])]
-        decisions.local_then_remote(path, local_diff, remote_diff, conflict=True, strategy=strategy)
-    elif remote_diff is ParentDeleted:
-        # Add marker at top of cell, easier to clean up manually
-        remote_diff = [op_addrange(0, ["<<<<<<< REMOTE CELL DELETED >>>>>>>\n"])]
-        decisions.remote_then_local(path, local_diff, remote_diff, conflict=True, strategy=strategy)
-    else:
-        # This is another approach, replacing content with markers
-        # if local_diff is ParentDeleted:
-        #     local = "<CELL DELETED>"
-        # else:
-        #     local = patch(base, local_diff)
-        # if remote_diff is ParentDeleted:
-        #     remote = "<CELL DELETED>"
-        # else:
-        #     remote = patch(base, remote_diff)
-
-        # Run merge renderer on full sources
-        local = patch(base, local_diff)
-        remote = patch(base, remote_diff)
-        merged, status = merge_render(base, local, remote, None)
-        conflict = status != 0
-
-        assert path[-1] == "source"
-        custom_diff = [op_replace(path[-1], merged)]
-        decisions.custom(path[:-1],
-            [op_patch(path[-1], local_diff)],
-            [op_patch(path[-1], remote_diff)],
-            custom_diff, conflict=conflict, strategy=strategy)
-
-    return decisions
-
-
-def resolve_strategy_generic(path, decisions, strategy):
-    if not (strategy and strategy != "mergetool" and decisions.has_conflicted()):
-        return
-
-    if strategy.startswith("use-"):
-        # Example: /cells strategy use-local,
-        # patch/remove conflict on /cells/3,
-        # decision is changed to action=local
-        action = strategy.replace("use-", "")
-        for d in decisions:
-            # Resolve conflicts that aren't marked with an
-            # already applied strategy. This applies to
-            # at least the inline conflict strategies.
-            if d.conflict and not d.get("strategy"):
-                d.action = action
-                d.conflict = False
-    else:
-        msg = "Unexpected strategy {} on {}.".format(
-            strategy, join_path(path))
-        nbdime.log.error(msg)
-
-
-def resolve_conflicted_decisions_list(path, base, decisions, strategy):
-    if not (strategy and strategy != "mergetool" and decisions.has_conflicted()):
-        return
-
-    if strategy == "inline-outputs":
-        # Affects conflicts on output dicts at /cells/*/outputs
-        resolve_strategy_inline_outputs(path, base, decisions)
-
-    elif strategy == "remove":
-        # Affects conflicts on output dicts at /cells/*/outputs
-        resolve_strategy_remove_outputs(path, base, decisions)
-
-    elif strategy == "clear-all":
-        # Old approach that relies on special handling in apply_decisions
-        # to deal with clear-all overriding other decisions:
-        # Collect local diffs and remote diffs from unresolved_conflicts
-        #local_conflict_diffs, remote_conflict_diffs = collect_conflicting_diffs(path, decisions)
-        #decisions.add_decision(path, "clear_all", local_conflict_diffs, remote_conflict_diffs, conflict=False)
-
-        # Just drop all decisions and add a decision to remove the entire range
-        local_diff, remote_diff = collect_diffs(path, decisions)
-        custom_diff = [op_removerange(0, len(base))]
-        decisions.decisions = []
-        decisions.custom(path, local_diff, remote_diff, custom_diff, conflict=False, strategy=strategy)
-
-    elif strategy == "clear":
-        # FIXME: Only getting here when base is a list of the lines in a string,
-        # actually resolved in resolve_conflicted_decisions_strings
-        # Avoid error message in generic
-        pass
-
-    else:
-        resolve_strategy_generic(path, decisions, strategy)
-
-
-def resolve_conflicted_decisions_dict(path, base, decisions, strategy):
-    if not (strategy and strategy != "mergetool" and decisions.has_conflicted()):
-        return
-
-    if strategy == "record-conflict":
-        # affects conflicts on dicts at /***/metadata or below
-        resolve_strategy_record_conflicts(path, base, decisions)
-
-    elif strategy == "inline-attachments":
-        # affects conflicts on string at /cells/*/attachments or below
-        resolve_strategy_inline_attachments(path, base, decisions)
-
-    else:
-        resolve_strategy_generic(path, decisions, strategy)
-
-
-def resolve_conflicted_decisions_strings(path, decisions, strategy):
-    if not (strategy and strategy != "mergetool" and decisions.has_conflicted()):
-        return
-
-    if strategy == "clear":
-        for d in decisions:
-            if d.conflict and not d.get("strategy"):
-                d.action = "clear"
-                d.conflict = False
-    elif strategy == "inline-source":
-        # Avoid error message in generic
-        pass
-    else:
-        resolve_strategy_generic(path, decisions, strategy)
-
-
-# Thinking through conflict resolution:
-# - when conflict first encountered, it's registered as dec.conflict(path, ..., strategy)
-# - strategy is used in tryresolve and eventually dropped if a conflict is registered
-# - when recursing and subdecisions are returned, they're added to decisions at this level
-# - i.e. decisions at this point contains all conflicts not resolved for subdocument,
-#   as well as unresolved conflicts for this level
-# - if we have a strategy at this level, we can try to resolve all
-#   conflicts of subdocument as well as those at this level
-
 
 # =============================================================================
 #
@@ -702,10 +200,6 @@ def _merge_dicts(base, local_diff, remote_diff, path, parent_decisions, strategi
 
     decisions = MergeDecisionBuilder()
 
-    # Intermediate list of conflicts recorded during first pass over diffs,
-    # resolution will be attempted at the end here
-    conflicts = []
-
     local_diff = as_dict_based_diff(local_diff)
     remote_diff = as_dict_based_diff(remote_diff)
 
@@ -822,9 +316,10 @@ def _split_addrange(key, local, remote, path, item_strategy):
     # First, find diff between local and remote insertion values.
     # This will align common subsequences according to the similarity
     # measures defined in notebook predicates.
-    intermediate_diff = diff(local, remote, path=star_path(path),
-                             predicates=notebook_predicates.copy(),
-                             differs=notebook_differs.copy())
+    intermediate_diff = perform_diff(
+        local, remote, path=star_path(path),
+        predicates=notebook_predicates.copy(),
+        differs=notebook_differs.copy())
 
     # Next, translate the diff into decisions
     decisions = MergeDecisionBuilder()
@@ -891,7 +386,8 @@ def _split_addrange(key, local, remote, path, item_strategy):
 
         elif d.op == DiffOp.PATCH:
             # Predicates indicate that local and remote items are similar!
-            decisions.conflict(path,
+            decisions.conflict(
+                path,
                 [op_addrange(key, [local[d.key]])],
                 [op_addrange(key, [remote[d.key + offset]])],
                 item_strategy)
@@ -935,9 +431,12 @@ def _merge_concurrent_inserts(base, ldiff, rdiff, path, decisions, item_strategy
     assert len(ldiff) == 1 or ldiff[1].op == DiffOp.REMOVERANGE
     assert len(rdiff) == 1 or rdiff[1].op == DiffOp.REMOVERANGE
 
-    subdecisions = _split_addrange(ldiff[0].key,
-        ldiff[0].valuelist, rdiff[0].valuelist,
-        path, item_strategy)
+    subdecisions = _split_addrange(
+        ldiff[0].key,
+        ldiff[0].valuelist,
+        rdiff[0].valuelist,
+        path,
+        item_strategy)
 
     # If there are any conflicts in the merging, and removals following...
     if subdecisions.has_conflicted() and (len(ldiff) == 2 or len(rdiff) == 2):
@@ -996,11 +495,11 @@ def _merge_lists(base, local_diff, remote_diff, path, parent_decisions, strategi
 
         # Build chunk names, easier to compare these strings
         # below than inspecting the diffs again and again
-        laname, lpname = chunkname(d0)
-        raname, rpname = chunkname(d1)
+        laname, lpname = chunk_typename(d0)
+        raname, rpname = chunk_typename(d1)
         lname = laname + lpname
         rname = raname + rpname
-        # /, A/, /A, A/A depending on addrange local and/or remote: 
+        # /, A/, /A, A/A depending on addrange local and/or remote:
         achunktype = laname + "/" + raname
         # Combinations of P (patch), R (removerange), or '',
         # e.g. P/R for local patch, remote removerange
@@ -1075,11 +574,10 @@ def _merge_lists(base, local_diff, remote_diff, path, parent_decisions, strategi
                     item_path, decisions, strategies)
                 decisions.extend(subdecisions)
             else:  # P/R or R/P
-                # Recurse into patches but with ParentDeleted sentinel passed instead of the diff
+                # Recurse into patches
                 if p0[0].op == DiffOp.PATCH:
                     thediff = p0[0].diff
                 elif p1[0].op == DiffOp.PATCH:
-                    ldiff = ParentDeleted
                     thediff = p1[0].diff
                 else:
                     nbdime.log.error("Unexpected op combination at this point.")
@@ -1115,7 +613,8 @@ def _merge_lists(base, local_diff, remote_diff, path, parent_decisions, strategi
                 elif will_diff_counter_parent_deletion(thediff, item_path, strategies):
                     # Keep the deleted item and instead let strategies for
                     # subobjects record that there has been a conflict
-                    counterdiff = create_parent_deletion_counter_diff(thediff, item_path, strategies)
+                    counterdiff = create_parent_deletion_counter_diff(
+                        thediff, item_path, strategies)
                     if p0[0].op == DiffOp.REMOVERANGE:
                         ld, rd = counterdiff, thediff
                     else:
@@ -1159,7 +658,7 @@ def _merge_lists(base, local_diff, remote_diff, path, parent_decisions, strategi
                 base, d0, d1, path, decisions, item_strategy)
             decisions.extend(subdecisions)
         else:
-            assert nbdime.log.error("Unhandled chunk conflict type %s" % (chunktype,))
+            assert nbdime.log.error("Unhandled chunk conflict type %s", chunktype)
 
     # Resolve remaining conflicts with strategy at this level if any
     #resolve_transients(path, base, decisions, transients)
@@ -1189,7 +688,8 @@ def _merge_strings(base, local_diff, remote_diff,
 
         # Mark as a conflict on parent (line):
         linenumber = path[-1]
-        decisions.conflict(path[:-1],
+        decisions.conflict(
+            path[:-1],
             [op_patch(linenumber, local_diff)],
             [op_patch(linenumber, remote_diff)],
             strategy)
@@ -1225,9 +725,9 @@ def _merge_strings(base, local_diff, remote_diff,
         resolve_conflicted_decisions_strings(path, decisions, strategy)
 
     if any([decisions.decisions[i] == decisions.decisions[j]
-           for i in range(len(decisions.decisions))
-           for j in range(len(decisions.decisions))
-           if i != j]):
+            for i in range(len(decisions.decisions))
+            for j in range(len(decisions.decisions))
+            if i != j]):
         nbdime.log.error("Found duplicated decisions, most likely a bug!")
 
     return decisions
@@ -1262,8 +762,8 @@ def decide_merge_with_diff(base, local, remote, local_diff, remote_diff, strateg
 
     parent_decisions = MergeDecisionBuilder()
 
-    decisions = _merge(base,
-        local_diff, remote_diff,
+    decisions = _merge(
+        base, local_diff, remote_diff,
         path, parent_decisions, strategies)
 
     # This is a remnant of the previous design, I think we
@@ -1390,6 +890,6 @@ def decide_merge(base, local, remote, strategies=None):
     """
     if strategies is None:
         strategies = Strategies({})
-    local_diff = diff(base, local)
-    remote_diff = diff(base, remote)
+    local_diff = perform_diff(base, local)
+    remote_diff = perform_diff(base, remote)
     return decide_merge_with_diff(base, local, remote, local_diff, remote_diff, strategies)
