@@ -16,7 +16,7 @@ import nbformat
 import nbdime
 import nbdime.log
 from nbdime.merging import merge_notebooks
-from nbdime.prettyprint import pretty_print_merge_decisions
+from nbdime.prettyprint import pretty_print_merge_decisions, PrettyPrintConfig
 from nbdime.utils import EXPLICIT_MISSING_FILE, read_notebook, setup_std_streams
 
 _description = ('Merge two Jupyter notebooks "local" and "remote" with a '
@@ -27,11 +27,14 @@ def main_merge(args):
     bfn = args.base
     lfn = args.local
     rfn = args.remote
-    mfn = args.output
+    mfn = args.out
+
+    from .args import process_diff_flags
+    process_diff_flags(args)
 
     for fn in (bfn, lfn, rfn):
         if not os.path.exists(fn) and fn != EXPLICIT_MISSING_FILE:
-            nbdime.log.error("Cannot find file '{}'".format(fn))
+            nbdime.log.error("Cannot find file '%s'", fn)
             return 1
 
     if lfn == rfn == EXPLICIT_MISSING_FILE:
@@ -57,14 +60,14 @@ def main_merge(args):
 
     if args.decisions:
         # Print merge decisions (including unconflicted)
-        out = io.StringIO()
-        pretty_print_merge_decisions(b, decisions, out=out)
-        nbdime.log.warning("Decisions:\n%s", out.getvalue())
+        config = PrettyPrintConfig(out=io.StringIO())
+        pretty_print_merge_decisions(b, decisions, config=config)
+        nbdime.log.warning("Decisions:\n%s", config.out.getvalue())
     elif mfn:
         # Write partial or fully completed merge to given foo.ipynb filename
-        with io.open(mfn, "w", encoding="utf8") as mf:
+        with io.open(mfn, "w", encoding="utf8"):
             nbformat.write(merged, mfn)
-        nbdime.log.info("Merge result written to %s" % mfn)
+        nbdime.log.info("Merge result written to %s", mfn)
     else:
         # Write merged notebook to terminal
         nbformat.write(merged, sys.stdout)
@@ -73,7 +76,8 @@ def main_merge(args):
 
 def handle_agreed_deletion(base_fn, output_fn, print_decisions=False):
     """Handle merge when file has been deleted both locally and remotely"""
-    assert base_fn != EXPLICIT_MISSING_FILE
+    assert base_fn != EXPLICIT_MISSING_FILE, (
+        'sanity check failed: cannot have agreed decision on base %r' % base_fn)
     b = read_notebook(base_fn, on_null='minimal')
     if print_decisions:
         # Print merge decision (delete all)
@@ -86,15 +90,15 @@ def handle_agreed_deletion(base_fn, output_fn, print_decisions=False):
         bld.agreement([], local_diff=diff, remote_diff=diff)
         decisions = bld.validated(b)
         # Print decition
-        out = io.StringIO()
-        pretty_print_merge_decisions(b, decisions, out=out)
+        config = PrettyPrintConfig(out=io.StringIO())
+        pretty_print_merge_decisions(b, decisions, config=config)
         nbdime.log.warning("Decisions:\n%s", out.getvalue())
 
     elif output_fn:
         # Delete file if existing, if not do nothing
         if os.path.exists(output_fn):
             os.remove(output_fn)
-            nbdime.log.info("Output file deleted: %s" % output_fn)
+            nbdime.log.info("Output file deleted: %s", output_fn)
 
 
 def _build_arg_parser():
@@ -110,13 +114,13 @@ def _build_arg_parser():
     add_filename_args(parser, ["base", "local", "remote"])
 
     parser.add_argument(
-        '-o', '--output',
+        '--out',
         default=None,
         help="if supplied, the merged notebook is written "
              "to this file. Otherwise it is printed to the "
              "terminal.")
     parser.add_argument(
-        '-d', '--decisions',
+        '--decisions',
         action="store_true",
         help="print a human-readable summary of conflicted "
              "merge decisions instead of merging the notebook.")
@@ -128,9 +132,7 @@ def main(args=None):
     if args is None:
         args = sys.argv[1:]
     setup_std_streams()
-    nbdime.log.init_logging()
     arguments = _build_arg_parser().parse_args(args)
-    nbdime.log.init_logging(level=arguments.log_level)
     return main_merge(arguments)
 
 
