@@ -130,7 +130,7 @@ def test_git_diff_driver_ignore_flags(filespath, capsys, nocolor, needs_git, res
 
 
 @pytest.mark.timeout(timeout=WEB_TEST_TIMEOUT)
-def test_git_web_diff_driver(filespath, unique_port):
+def test_git_web_diff_driver(filespath, unique_port, reset_log):
     # Simulate a call from `git diff` to check basic driver functionality
 
     fn1 = os.path.join(filespath, 'foo--1.ipynb')
