@@ -10,6 +10,7 @@ import os
 import re
 from subprocess import check_output, CalledProcessError
 import sys
+from contextlib import contextmanager
 
 import nbformat
 from six import string_types, text_type, PY2
@@ -20,17 +21,18 @@ else:
     EXPLICIT_MISSING_FILE = '/dev/null'
 
 
-def read_notebook(filename, on_null):
+def read_notebook(f, on_null):
     """Read and return notebook json from filename
 
     Parameters:
-        filename: The filename to read from or null filename
-            ("/dev/null" on *nix, "nul" on Windows)
+        f:  The filename to read from or null filename
+            ("/dev/null" on *nix, "nul" on Windows).
+            Alternatively a file-like object can be passed.
         on_null: What to return when filename null
             "empty": return empty dict
             "minimal": return miminal valid notebook
     """
-    if filename == EXPLICIT_MISSING_FILE:
+    if f == EXPLICIT_MISSING_FILE:
         if on_null == 'empty':
             return {}
         elif on_null == 'minimal':
@@ -40,7 +42,7 @@ def read_notebook(filename, on_null):
                 'Not valid value for `on_null`: "%s". Valid values '
                 'are "empty" or "minimal"', on_null)
     else:
-        return nbformat.read(filename, as_version=4)
+        return nbformat.read(f, as_version=4)
 
 
 def as_text(text):
@@ -56,8 +58,9 @@ def as_text_lines(text):
         text = text.splitlines(True)
     if isinstance(text, tuple):
         text = list(text)
-    assert isinstance(text, list)
-    assert all(isinstance(t, string_types) for t in text)
+    assert isinstance(text, list), 'text argument should be string or string sequence'
+    assert all(isinstance(t, string_types) for t in text), (
+        'text argument should be string or string sequence')
     return text
 
 
@@ -245,7 +248,7 @@ def find_shared_prefix(a, b):
 
 def _setup_std_stream_encoding():
     """Setup encoding on stdout/err
-    
+
     Ensures sys.stdout/err have error-escaping encoders,
     rather than raising errors.
     """
@@ -273,7 +276,7 @@ def _setup_std_stream_encoding():
 
 def setup_std_streams():
     """Setup sys.stdout/err
-    
+
     - Ensures sys.stdout/err have error-escaping encoders,
       rather than raising errors.
     - enables colorama for ANSI escapes on Windows
@@ -286,3 +289,17 @@ def setup_std_streams():
         import colorama
         colorama.init()
 
+
+def split_os_path(path):
+    return os.path.normpath(path).lstrip(os.path.sep).split(os.path.sep)
+
+
+@contextmanager
+def pushd(path):
+    """Change current directory with context manager (changes back)"""
+    old = os.curdir
+    try:
+        os.chdir(path)
+        yield
+    finally:
+        os.chdir(old)
