@@ -9,10 +9,9 @@ from argparse import ArgumentParser
 
 from ..args import (
     add_generic_args, add_diff_args, add_web_args, add_filename_args,
-    args_for_server, args_for_browse)
+    args_for_server, args_for_browse, process_diff_flags)
 from .nbdimeserver import main_server as run_server
 from .webutil import browse
-import nbdime.log
 
 
 # TODO: Tool server is passed a (mandatory?) single-use access token, which is
@@ -41,7 +40,7 @@ def main_parsed(opts):
 
     Called by both main here and gitdifftool
     """
-    nbdime.log.init_logging(level=opts.log_level)
+    process_diff_flags(opts)
     base = opts.local
     remote = opts.remote
     return run_server(
