@@ -27,6 +27,7 @@ from nbdime.args import add_generic_args, add_web_args
 from notebook.base.handlers import IPythonHandler, APIHandler
 from notebook import DEFAULT_STATIC_FILES_PATH
 from notebook.utils import url_path_join
+from notebook.log import log_request
 
 # TODO: See <notebook>/notebook/services/contents/handlers.py for possibly useful utilities:
 #@json_errors
@@ -119,6 +120,14 @@ class NbdimeHandler(IPythonHandler):
     def curdir(self):
         return self.params.get('cwd', os.curdir)
 
+    # Polyfill in until we can rely on Notebook 5.0 being present
+    @property
+    def mathjax_config(self):
+        try:
+            return super(NbdimeHandler, self).mathjax_config
+        except AttributeError:
+            return 'TeX-AMS_HTML-full,Safe'
+
 
 class MainHandler(NbdimeHandler):
     def get(self):
@@ -223,6 +232,7 @@ class ApiDiffHandler(NbdimeHandler, APIHandler):
             arg = self.params['difftool_args'][argname]
             if not isinstance(arg, string_types):
                 # Assume arg is file-like
+                arg.seek(0)
                 return nbformat.read(arg, as_version=4)
             return self.read_notebook(arg)
         return super(ApiDiffHandler, self).get_notebook_argument(argname)
@@ -326,6 +336,7 @@ def make_app(**params):
 
     env = Environment(loader=FileSystemLoader([template_path]), autoescape=False)
     settings = {
+        'log_function': log_request,
         'static_path': static_path,
         'static_url_prefix': prefix + '/static/',
         'template_path': [template_path],
@@ -354,7 +365,7 @@ def init_app(on_port=None, closable=False, **params):
     port = params.pop('port', 0)
     ip = params.pop('ip', '127.0.0.1')
     app = make_app(**params)
-    if port != 0 or on_port is None:
+    if port != 0:
         server = app.listen(port, address=ip)
         _logger.info('Listening on %s, port %d', ip, port)
     else:
@@ -371,7 +382,14 @@ def init_app(on_port=None, closable=False, **params):
 
 def main_server(on_port=None, closable=False, **params):
     app, server = init_app(on_port, closable, **params)
-    ioloop.IOLoop.current().start()
+    io_loop = ioloop.IOLoop.current()
+    if sys.platform.startswith('win'):
+        # workaround for tornado on Windows:
+        # add no-op to wake every 5s
+        # to handle signals that may be ignored by the inner loop
+        pc = ioloop.PeriodicCallback(lambda : None, 5000)
+        pc.start()
+    io_loop.start()
     # Clean up after server:
     server.stop()
     return app.exit_code
